unit DbgTimer;

(*                                                                             *
  Name:      DbgTimer                                                          *
                                                                               *
  Objective: Simplified use of precision timer/stopwatch especially for        *
             program development/testing/debugging with Lazarus/FPC.           *
                                                                               *
  Author:    Ruediger Walter <rue.walter@web.de>                               *
                                                                               *
  Version:   0.7.6 / 2014-09-26                                                *
                                                                               *
  Features:  DbgTimer provides a specific and simplified use of EpikTimer,     *
             especially designed for developing and analyzing time-critical    *
             sections and loops in Free Pascal programs/units using the        *
             Lazarus IDE.                                                      *
             For this purpose EpikTimer ist the appropriate tool. Please       *
             keep in mind, that Epiktimer is a virtual stop-watch, while       *
             the LCL-own TTimer (or TFPTimer) are virtual alarm-clocks         *
             (respectively virtual metronomes). Hence EpicTimer doesn't        *
             consume own computing ressources during the measured loop and     *
             has not to deal with tick-event notification overhead.            *
             DbgTimer avoids the incorporation of the EpikTimer LCL component  *
             into the project and can be removed easily afterwards. There's    *
             no need to install the EpikTimer package into the Lazarus IDE,    *
             it only requires the EpikTimer unit to be placed into the         *
             FPC/Lazarus searchpath. For further information on EpikTimer      *
             please take a look at http://wiki.freepascal.org/EpikTimer        *
                                                                               *
  Usage:     1) Put "DbgTimer" into the uses-part of the FPC/Lazarus-unit      *
                you want to explore.                                           *
                                                                               *
             2) There are only three (and a half) procedure/function calls:    *
                "DebugTimerStart", "DebugTimerStop" and "DebugTimerPause()".   *
                Just place the start/stop calls around the section or loop     *
                you want to investigate, like this:                            *
                >   DebugTimerStart;                                           *
                >   for i := 0 to FFiles.Count - 1 do ParseFile(i);            *
                >   DebugTimerStop;                                            *
                >   NextInstruction;           // Place a Breakpoint here      *
                                                                               *
                DebugTimerStart will reset and start the virtual stopwatch,    *
                DebugTimerStop (surprisingly) stops it and calculates the      *
                results.                                                       *
                                                                               *
                If you want to know the per cycle time consumption, you can    *
                call "DebugTimerStop" with an integer parameter instead:       *
                >   DebugTimerStop({Number of Cycles, in this case:} i+1);     *
                                                                               *
                Usually you might use "DebugTimerStop" like a procedure        *
                without reading the function result, but if you need the       *
                numerical (Extended type) value of the total time used (in     *
                seconds) for some further calculations, you can use its        *
                function result:                                               *
                >   MyLameDucksDawdleTime := DebugTimerStop;                   *
                                                                               *
                If you want to pause and continue without reset, just call     *
                >   DebugTimerPause(true);                                     *
                >   DoSomethingIrrelevant;                                     *
                >   DebugTimerPause(false);                                    *
                To pause the stopwatch doesn't make much sense if it's not     *
                yet started, so DebugTimerPause will raise an exception on     *
                this attempt (which is better than raising confusion with      *
                fantastic results...).                                         *
                To pause can be useful if you want to extract a particular     *
                sequence, e.g. a nested loop. But keep in mind: The smaller    *
                your time fragment, the less accurate your measure and the     *
                larger the burden of additional jumps and system calls will    *
                be. Thus adding lots of nanosecond intervals will not make     *
                much sense (see also "Limits").                                *
                                                                               *
             3) Go to the DbgTimer unit (e.g. by placing the cursor on         *
                "DebugTimerStart" and doing ALT+Up), click on the              *
                "DebugTimeTotal" and "DebugTimePerCycle" variables and         *
                add them to the Lazarus IDE watch list (Strg+F5).              *
                                                                               *
             4) Now put a breakpoint to the next line BELOW "DebugTimerStop".  *
                When running your program, the debugger will pause the         *
                execution after time measuring and you can read the values     *
                in the watch list, like this:                                  *
                >   DebugTimeTotal       '0.217390 s'                          *
                >   DebugTimePerCycle    '1.302 ms   (167 Cycles)'             *
                                                                               *
             5) To remove DbgTimer when the work is done just kick "DbgTimer"  *
                out of the "uses"-line, the compiler will help you to find     *
                all residues of your examination.                              *
                                                                               *
  Limits:    EpikTimer is a really nice virtual stopwatch and DebugTimer might *
             be a useful tool to optimize your source code (I hope at least).  *
             But you should have an idea about the limits of time measuring    *
             on the computer. It's very useless to stare at the amazing nano-  *
             seconds measuring abilities of the latest, best, most expensive,  *
             most accurate processor family while working on a multitasking,   *
             non-realtime operating system. If you press the stop-button e.g.  *
             via a key stroke, your virtual stopwatch won't stop immediately,  *
             but at the moment when the OS decides to execute the instruction. *
             An average Linux computer typically needs 1-3 milliseconds to     *
             respond if another task is in charge at that moment. In return    *
             the execution of your program will be frequently interrupted by   *
             some task (or thread) switching. This is not due to a lack of     *
             accuracy of the CPU or of EpikTimer - those are simply the rules  *
             of your operating system. Thus measuring very short periods will  *
             decrease the likelihood of interrupts, but might bring the        *
             measure near to the accuracy limits of the CPU, and it will       *
             increase the impact of scattered interruptions - and vice versa.  *
             That's why your results will always vary a little bit. Measuring  *
             time periods > 10 ms will produce reasonably reliable results     *
             (mostly), but you should never rely on a single measure. If you   *
             need very high accuracy when fine-tuning some code snippets,      *
             you'd better test them in loops > 10 seconds in order to level    *
             out random running conditions.                                    *
                                                                               *
  Credits:   Many thanks to Tom Lisjac, Felipe Monteiro de Carvalho, Marcel    *
             Minderhoud and Graeme Geldenhuys for developing, maintaining      *
             and providing EpikTimer. Actually this tiny unit here is nearly   *
             exclusively their merit. DbgTimer only provides a specific use    *
             of EpikTimer, so the real work was done by Tom, Felipe, Marcel    *
             and Graeme.                                                       *
                                                                               *
  Copyright (C) 2014 by Ruediger Walter <rue.walter@web.de>                    *
                                                                               *
  This library is licensed on the same Modifyed LGPL as Free Pascal RTL and    *
  Lazarus LCL. See the file COPYING.FPC, included in your FPC/Lazarus          *
  distribution, for details about the copyright.                               *
                                                                               *
  This program is distributed in the hope that it will be useful, but WITHOUT  *
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
  FITNESS FOR A PARTICULAR PURPOSE.                                            *)



interface

uses
  Classes, SysUtils, EpikTimer;


procedure DebugTimerStart;
procedure DebugTimerPause(DoPause: Boolean);
function  DebugTimerStop: Extended; overload;
function  DebugTimerStop(NoOfCycles: integer): Extended; overload;

var
  DebugTimeTotal,                           // Add those variables to the
  DebugTimePerCycle : String;               // Lazarus IDE watch list (Strg+F5)


implementation

var
  DT       : TEpikTimer;
  Running  : Boolean;

const
  Prec_ms = 4;        // Decimal places Milliseconds      |  Modify if you like
  Prec_s  = 3;        // Decimal places Seconds           |  another resolution

  ErrMsg  = 'You must start DebugTimer before pause';


procedure DebugTimerStart;
begin
  DT.Clear;
  Running := true;
  DT.Start;
end;

procedure DebugTimerPause(DoPause : Boolean);
begin
  if not Running then
    raise exception.create(ErrMsg) at get_caller_addr(get_frame),
                                      get_caller_frame(get_frame);
  if DoPause then DT.Stop
  else DT.Start;
end;

function DebugTimerStop(NoOfCycles: integer=0): Extended;
var
  T : Extended;
  Cycles : String;
begin
  DT.Stop;
  Running := false;
  Result := DT.Elapsed;
  DebugTimeTotal := DT.ElapsedStr + ' s';
  if NoOfCycles <= 0 then DebugTimePerCycle := 'Number of cycles not specified'
  else begin
    Cycles := '   (' + IntToStr(NoOfCycles) + ' Cycles)';
    T := DT.Elapsed / NoOfCycles;
    if Result < 0.1 then
    begin
      T := T * 1000;
      DebugTimePerCycle := format('%.'+IntToStr(Prec_ms)+'f',[T]) + ' ms' + Cycles;
    end
    else DebugTimePerCycle := format('%.'+IntToStr(Prec_s)+'f',[T]) + ' s' + Cycles;
  end;
end;

function DebugTimerStop: Extended;
begin
  Result := DebugTimerStop(0);
end;


initialization
  DT := TEpikTimer.Create(nil);
  DT.StringPrecision := Prec_s;
  Running := false;

finalization
  DT.Free;

end.

